#!/bin/bash
#
# 1-wire Li-Ion charger/discharger
#
# (C) 2006 Marek Wodzinski
# http://majek.mamy.to
#


LOG=ladowarka.log	#logs
CAL=ladowarka.cal	#file with calibration

VOLT_ADR=20.48B706000000	#1w address of DS2450 ADC
VOLT_BAT_O=volt.A		#pin configurations
VOLT_CHAR_O=volt2.B
VOLT_DISC_O=volt.C
VOLT_ZERO_O=volt2.D
VOLT_BAT=1
VOLT_CHAR=2
VOLT_DISC=3
VOLT_ZERO=4
SW_ADR=3A.114000000000		#1w address of DS2413 switch
SW_CHAR=0			#put 0 to PIO.BYTE for charge
SW_DISC=3			#...
SW_OFF=2
OW=/mnt/1wire/uncached		#1-wire filesystem root directory

R_DISC=9.45	#Ohm (value of R8)
R_CHAR=0.458	#Ohm (value of R1)
I_CHAR_MIN=50	#mA  (minimum current to stop charging)
U_DISC_MIN=2750	#mV  (minumum voltage to stop discharge)

DONE=0
STATE=U
STATE_NEXT_C=W	#next state after charge
STATE_NEXT_D=C	#next state after discharge
WAIT_TIME=3600	#wait time between charge and discharge (in seconds)
MAH_UP=0
MAH_DOWN=0
WAITT=0

function log
{
    echo "`date +%F'/'%T` $STATE $1" >>$LOG 
}

function przelicz
{
    x10k=`echo -e "$1*10000\n"|bc|cut -f 1 -d '.'`
    while read wspol; do
      c10k=`echo $wspol|cut -f 1 -d ':'`
      if [ $c10k -gt $x10k ]; then
        a=`echo $wspol|cut -f 2 -d ':'`
        b=`echo $wspol|cut -f 3 -d ':'`
	break
      fi
    done <$CAL
    echo -e "scale=6\n($a)*$1+($b)\n"|bc
}

function newstate
{
    log "-> $1 $MAH_UP $MAH_DOWN"
    case $1 in
	O)
	    echo $SW_OFF >$OW/$SW_ADR/PIO.BYTE
	    STATE='O'
	    Ichar=0
	    Idisc=0
	    ;;
	D)
	    echo $SW_DISC >$OW/$SW_ADR/PIO.BYTE
	    STATE='D'
	    MAH_DOWN=0
	    Ichar=0
	    ;;
	C)
	    echo $SW_CHAR >$OW/$SW_ADR/PIO.BYTE
	    STATE='C'
	    MAH_UP=0
	    Idisc=0
	    ;;
	W)
	    echo $SW_OFF >$OW/$SW_ADR/PIO.BYTE
	    STATE='W'
	    WAITT=$((`date +%s`+$WAIT_TIME))
	    Idisc=0
	    Ichar=0
	    ;;
    esac
    ladav=`cat $OW/$VOLT_ADR/$VOLT_CHAR_O`
    rozlav=`cat $OW/$VOLT_ADR/$VOLT_DISC_O`
}



case "$1" in
     off)
        newstate 'O'
        ;;
     d*)
        newstate 'D'
	;;
     c*)
	newstate 'C'
	;;
     *)
        echo "ladowarka.sh: undefined action!"
	echo "Usage: ladowarka.sh c|d|o [cycles]"
	exit
	;;
esac

if [ -z "$2" ]; then
  CYCLES=999
else
  CYCLES=$(($2))
fi


#init values
batav=`cat $OW/$VOLT_ADR/$VOLT_BAT_O`
ladav=`cat $OW/$VOLT_ADR/$VOLT_CHAR_O`
zeroav=`cat $OW/$VOLT_ADR/$VOLT_ZERO_O`
rozlav=`cat $OW/$VOLT_ADR/$VOLT_DISC_O`
curtime=`date +%s`

while [ $DONE -eq 0 ]; do
  sleep 1
  napiecia2=`cat $OW/$VOLT_ADR/volt2.ALL`
  sleep 1
  napiecia=`cat $OW/$VOLT_ADR/volt.ALL`
  
  bateria=`echo "$napiecia2"|cut -f $VOLT_BAT -d ','`
  if [ $bateria = 0 ]; then
    bateria=`echo "$napiecia"|cut -f $VOLT_BAT -d ','`
  fi
  if [ $bateria = 0 ]; then
    bateria=5.12
  fi
  if [ -z "`echo $bateria`" ]; then
    bateria=$batav
  fi

  batav=`echo -e "scale=5\n($batav*3+$bateria)/4\n"|bc`
  batreal=`przelicz "$batav"`
  
  ladowanie=`echo "$napiecia2"|cut -f $VOLT_CHAR -d ','`
  if [ $ladowanie = 0 ]; then
    ladowanie=`echo "$napiecia"|cut -f $VOLT_CHAR -d ','`
  fi
  if [ -z "`echo $ladowanie`" ]; then
    ladowanie=$ladav
  fi
  ladav=`echo -e "scale=5\n($ladav*3+$ladowanie)/4\n"|bc`
  ladreal=`przelicz "$ladav"`

  rozl=`echo "$napiecia2"|cut -f $VOLT_DISC -d ','`
  if [ $rozl = 0 ]; then
    rozl=`echo "$napiecia"|cut -f $VOLT_DISC -d ','`
  fi
  if [ -z "`echo $rozl`" ]; then
    rozl=$rozlav
  fi
  rozlav=`echo -e "scale=5\n($rozlav*3+$rozl)/4\n"|bc`
  rozlreal=`przelicz "$rozlav"`
  
  if [ $STATE = 'C' ]; then
    ubat=`echo -e "scale=5\n$batreal-($ladreal)\n"|bc`
  else
    ubat=$batreal
  fi
  
  lasttime=$curtime
  curtime=`date +%s`
  
  #states
  #C - charge
  #D - discharge
  #O - off
  #W - wait for discharge
  
  case $STATE in
    C)
	#calculate charge current
	Ichar=`echo -e "scale=0\n$ladreal*1000/$R_CHAR\n"|bc`
	#capacity - mAh
	MAH_UP=`echo -e "scale=6\n$MAH_UP+$Ichar*($curtime-$lasttime)/3600\n"|bc`
	if [ $Ichar -le $I_CHAR_MIN ]; then
	    #end of charging!
	    #check number of cycles
	    CYCLES=$(($CYCLES-1))
	    if [ $CYCLES -le 0 ]; then
	      newstate O
	    else
	      newstate $STATE_NEXT_C
	    fi
	fi
	;;
    D)
	#calculate discharge current
	Idisc=`echo -e "scale=0\n$rozlreal*1000/$R_DISC\n"|bc`
	#mAh
	MAH_DOWN=`echo -e "scale=6\n$MAH_DOWN+$Idisc*($curtime-$lasttime)/3600\n"|bc`
	ubat=`echo -e "scale=5\n$ubat+$Idisc*$R_CHAR/1000\n"|bc`		#correction to charge resistor (R1)
	tempu=`echo -e "$ubat*1000<$U_DISC_MIN\n"|bc`
	if [ $tempu -eq 1 ]; then
	    #end of discharging!
	    newstate $STATE_NEXT_D
	fi
	;;
    W)
	#waiting...
	if [ $curtime -gt $WAITT ]; then
	    newstate 'D'
	fi
	;;
  esac
  
  echo "U=$batreal | Uchar=$ladreal | Ichar=$Ichar | Udis=$rozlreal | Idisc=$Idisc | Ubat=$ubat | mAh_up=$MAH_UP | mAh_down=$MAH_DOWN"
  #you can uncomment second line to show raw values from ADC (useful for calibration)
  #echo "R:U=$bateria | Uchar=$ladowanie | Udis=$rozlreal || U=$batreal | Uchar=$ladreal | Udis=$rozlreal"
  log "U=$batreal Uchar=$ladreal Ichar=$Ichar Udis=$rozlreal Idisc=$Idisc Ubat=$ubat mAh_up=$MAH_UP mAh_down=$MAH_DOWN"
done
